#include <dbus/dbus-glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "bmpx/dbus.h"

int
main (int argc, char **argv)
{
  DBusGConnection *bus;
  DBusGProxy *remote_object;
  GError *error = NULL;

  g_type_init ();

  {
    GLogLevelFlags fatal_mask;
    
    fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
    fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
    g_log_set_always_fatal (fatal_mask);
  }

  bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
  if (!bus)
  {
  	g_log (G_LOG_DOMAIN,G_LOG_LEVEL_ERROR,"Couldn't connect to session bus: %s", error->message);
	exit (EXIT_FAILURE);
  }
  
  remote_object = dbus_g_proxy_new_for_name (bus,
					     BMP_DBUS_SERVICE,
					     BMP_DBUS_PATH_SYSTEMCONTROL,
					     BMP_DBUS_INTERFACE);

  /* FIXME: Provide a generic interface a la gst-launch: bmp-dbus-system-control SomeCommand int=5 string='blahblah' */
  if (! strcmp(argv[1], "Quit"))
  {
  	if (!dbus_g_proxy_call (remote_object, "Quit", &error,
  			  G_TYPE_INVALID, G_TYPE_INVALID))
		g_message(error->message);

	goto END;
  }

  if (! strcmp(argv[1], "UiStart"))
  {
  	if (!dbus_g_proxy_call (remote_object, "UiStart", &error,
  			  G_TYPE_INVALID, G_TYPE_INVALID))
		g_message(error->message);

	goto END;
  }

  if (! strcmp(argv[1], "UiStop"))
  {
  	if (!dbus_g_proxy_call (remote_object, "UiStop", &error,
  			  G_TYPE_INVALID, G_TYPE_INVALID))
		g_message(error->message);

	goto END;
  }

  if (! strcmp(argv[1], "PlayNext"))
  {
  	if (!dbus_g_proxy_call (remote_object, "PlayNext", &error,
  			  G_TYPE_INVALID, G_TYPE_INVALID))
		g_message(error->message);

	goto END;
  }

  if (! strcmp(argv[1], "PlayPrev"))
  {
  	if (!dbus_g_proxy_call (remote_object, "PlayPrev", &error,
  			  G_TYPE_INVALID, G_TYPE_INVALID))
		g_message(error->message);

	goto END;
  }

  if (! strcmp(argv[1], "PlayPause"))
  {
  	if (!dbus_g_proxy_call (remote_object, "PlayPause", &error,
  			  G_TYPE_INVALID, G_TYPE_INVALID))
		g_message(error->message);

	goto END;
  }

  if (! strcmp(argv[1], "PlayStop"))
  {
  	if (!dbus_g_proxy_call (remote_object, "PlayStop", &error,
  			  G_TYPE_INVALID, G_TYPE_INVALID))
		g_message(error->message);

	goto END;
  }

  if (! strcmp(argv[1], "PlayCurrent"))
  {
  	if (!dbus_g_proxy_call (remote_object, "PlayCurrent", &error,
  			  G_TYPE_INVALID, G_TYPE_INVALID))
		g_message(error->message);

	goto END;
  }

  if (! strcmp(argv[1], "PlayTrack"))
  {
	gint track;

	track = strtol (argv[2], NULL, 10);

  	if (!dbus_g_proxy_call (remote_object, "PlayTrack", &error,
			  G_TYPE_INT, track,
  			  G_TYPE_INVALID, G_TYPE_INVALID));

	goto END;
  }

  if (! strcmp(argv[1], "PlayListTrack"))
  {
	gint list;
        gint track;
	
	list  = strtol (argv[2], NULL, 10);
	track = strtol (argv[3], NULL, 10);
  	if (!dbus_g_proxy_call (remote_object, "PlayListTrack", &error,
			  G_TYPE_INT, list, 
			  G_TYPE_INT, track,
  			  G_TYPE_INVALID, G_TYPE_INVALID));

	goto END;
  }

  if (! strcmp(argv[1], "VolumeSet"))
  {
	gint volume;

	volume = strtol (argv[2], NULL, 10);

  	if (!dbus_g_proxy_call (remote_object, "VolumeSet", &error,
			  G_TYPE_INT, volume,
  			  G_TYPE_INVALID, G_TYPE_INVALID));

	goto END;
  }

  if (! strcmp(argv[1], "SetCurrentList"))
  {
	gint list;

	list = strtol (argv[2], NULL, 10);

  	if (!dbus_g_proxy_call (remote_object, "SetCurrentList", &error,
			  G_TYPE_INT, list,
  			  G_TYPE_INVALID, G_TYPE_INVALID));

	goto END;
  }


  if (! strcmp(argv[1], "Repeat"))
  {
	gint repeat;

	repeat = strtol (argv[2], NULL, 10);

  	if (!dbus_g_proxy_call (remote_object, "Repeat", &error,
			  G_TYPE_INT, repeat,
  			  G_TYPE_INVALID, G_TYPE_INVALID));

	goto END;
  }

  if (! strcmp(argv[1], "Seek"))
  {
	gint seek;

	seek = strtol (argv[2], NULL, 10);

  	if (!dbus_g_proxy_call (remote_object, "Seek", &error,
			  G_TYPE_INT, seek,
  			  G_TYPE_INVALID, G_TYPE_INVALID));

	goto END;
  }

  if (! strcmp(argv[1], "GetCurrentTitle"))
  {
	gchar *title;

	if (!dbus_g_proxy_call (remote_object, "GetCurrentTitle", &error,
			  G_TYPE_INVALID,
			  G_TYPE_STRING, &title, G_TYPE_INVALID));
 
	printf("%s\n",title);

	goto END;
  }

  if (! strcmp(argv[1], "GetCurrentUri"))
  {
	gchar *uri;

	if (!dbus_g_proxy_call (remote_object, "GetCurrentUri", &error,
			  G_TYPE_INVALID,
			  G_TYPE_STRING, &uri, G_TYPE_INVALID));
 
	printf("%s\n",uri);

	goto END;
  }

  if (! strcmp(argv[1], "AddUriList"))
  {
	gchar **uri_list;
	gint tracklist, position,n,rv;

	tracklist = atoi (argv[2]);
	position  = atoi (argv[3]);

	uri_list = g_new (gchar*, argc-3);

	for (n = 4; n < argc; n++)
          {
	    uri_list[n-4] = argv[n];
          }
	uri_list[n-4] = NULL;

#if 0
	uri_list[0] =
	    argv[4];
	uri_list[1] =
	    NULL;
#endif

	if (!dbus_g_proxy_call (remote_object, "AddUriList", &error,
			  G_TYPE_STRV, uri_list,
			  G_TYPE_INT, tracklist, 
			  G_TYPE_INT, position, 
			  G_TYPE_INVALID,
			  G_TYPE_INT, &rv,
			  G_TYPE_INVALID)); 
 
	goto END;
  }


  END:

  exit(EXIT_SUCCESS);
}
