/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include <glib.h>
#include <glib-object.h>

#include "bmpx/plugin.h"
#include "bmpx/playlist.h"
#include "bmpx/metadata.h"
#include "bmpx/list.h"
#include "bmpx/plugin_interfaces.h"

#include "libflow_linear.h"

/* Linear Flow Control */
static BmpPlaybackHistoryItem*
flow_linear_item_next (BmpPluginFlowInterface *self, const BmpListType *list, gint _idx)
{
    BmpPlaylistItem *playlist;
    BmpFileItem *file_item;
    BmpPlaybackHistoryItem *history_item;
    gint length;
    gint current;

    playlist = 
	BMP_PLAYLIST_ITEM(g_list_nth_data(list->items, _idx)); 

    length =
	g_list_length (playlist->list->items)-1;

    current =
	playlist->list->current;

    if (current < length)
      {
        history_item = 
	    g_new0 (BmpPlaybackHistoryItem,1);
	file_item =
	    BMP_FILE_ITEM(g_list_nth_data(playlist->list->items, current+1));

	history_item->list_time = 
	    playlist->list->time;

	history_item->list_name = 
	    g_strdup (playlist->listname);
    
	history_item->guid = g_malloc0(sizeof(RowGUID));
	memcpy (history_item->guid, file_item->guid, sizeof(RowGUID));
      }
    else
      {
	history_item = NULL;
      }

    return history_item;
}

static BmpPlaybackHistoryItem*
flow_linear_item_prev  (BmpPluginFlowInterface *self, const BmpListType *list, gint _idx)
{
    BmpPlaylistItem *playlist;
    BmpFileItem *file_item;
    BmpPlaybackHistoryItem *history_item;
    gint length;
    gint current;

    playlist = 
	BMP_PLAYLIST_ITEM(g_list_nth_data(list->items, _idx)); 

    length =
	g_list_length (playlist->list->items)-1;

    current =
	playlist->list->current;

    if (current > 0)
      {
        history_item = 
	    g_new0 (BmpPlaybackHistoryItem,1);
	file_item =
	    BMP_FILE_ITEM(g_list_nth_data(playlist->list->items, current-1));

	history_item->list_time = 
	    playlist->list->time;

	history_item->list_name = 
	    g_strdup (playlist->listname);

	history_item->guid = g_malloc0(sizeof(RowGUID));
	memcpy (history_item->guid, file_item->guid, sizeof(RowGUID));
      }
    else
      {
	history_item = NULL;
      }

    return history_item;
}

static gchar* _metadata[] = {
    "Linear",
    "Milosz Derezynski",
    "(C) GPL 2005",
    "http://bmpx.berlios.de",
    "internalerror@gmail.com",
};

const gchar**
get_metadata (BmpPlugMetadataInterface *self)
{
    return (const gchar**)_metadata;
}

static void
metadata_init (gpointer         g_iface,
               gpointer         iface_data)
{
        BmpPlugMetadataInterfaceClass *klass = (BmpPlugMetadataInterfaceClass *)g_iface;

        klass->get_metadata = get_metadata;
}


static void
flow_linear_interface_init (gpointer         g_iface,
                            gpointer         iface_data)
{
        BmpPluginFlowInterfaceClass *klass = (BmpPluginFlowInterfaceClass *)g_iface;

        klass->item_next = flow_linear_item_next;
        klass->item_prev = flow_linear_item_prev;
}

GType plugin_flow_linear_type = 0;

void 
plugin_flow_linear_register_type (GTypeModule *module)
{
	    static const GTypeInfo object_info = {
                        sizeof (BmpPluginFlowLinearClass),
                        NULL,   /* base_init */
                        NULL,   /* base_finalize */
                        NULL,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        sizeof (BmpPluginFlowLinear),
                        0,      /* n_preallocs */
                        NULL,    /* instance_init */
                };

                plugin_flow_linear_type = g_type_module_register_type (module,
					   G_TYPE_OBJECT,
					  "BmpPluginFlowLinearType",
					   &object_info, 0);

                static const GInterfaceInfo interface_flow_linear_info = {
                        (GInterfaceInitFunc) flow_linear_interface_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                static const GInterfaceInfo interface_metadata_info = {
                        (GInterfaceInitFunc) metadata_init,    /* interface_init */
                        NULL,                                       /* interface_finalize */
                        NULL                                        /* interface_data */
                };

                g_type_module_add_interface (module,
					     plugin_flow_linear_type,
                                             BMP_TYPE_PLUG_METADATA,
                                             &interface_metadata_info);

                g_type_module_add_interface (module,
					     plugin_flow_linear_type,
                                             BMP_TYPE_PLUGIN_FLOW_IFACE,
                                             &interface_flow_linear_info);
}
