/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */

#include <glib.h>
#include <glib-object.h>

#include "bmpx/plugin.h"
#include "bmpx/plugin_interfaces.h"
#include "bmpx/playlist.h"
#include "bmpx/metadata.h"
#include "bmpx/list.h"
#include "bmpx/vfs.h"

/* PlugMetadata IFace */
static void
bmp_plug_metadata_iface_base_init (gpointer g_class)
{
        static gboolean initialized = FALSE;

        if (!initialized) {
                /* create interface signals here. */
                initialized = TRUE;
        }
}

GType
bmp_plug_metadata_iface_get_type (void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (BmpPlugMetadataInterfaceClass),
                        bmp_plug_metadata_iface_base_init,   /* base_init */
                        NULL,   /* base_finalize */
                        NULL,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        0,
                        0,      /* n_preallocs */
                        NULL    /* instance_init */
                };
                type = g_type_register_static (G_TYPE_INTERFACE, "BmpPlugMetadataInterface", &info, 0);
        }
        return type;
}

const gchar**
bmp_plug_metadata_get (BmpPlugMetadataInterface *self)
{
	g_return_val_if_fail (BMP_IS_PLUG_METADATA(self), NULL);

        return BMP_PLUG_METADATA_GET_CLASS (self)->get_metadata (self);
}


/* Flow Plugin IFace */
static void
bmp_plugin_flow_iface_base_init (gpointer g_class)
{
        static gboolean initialized = FALSE;

        if (!initialized) {
                /* create interface signals here. */
                initialized = TRUE;
        }
}

GType
bmp_plugin_flow_iface_get_type (void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (BmpPluginFlowInterfaceClass),
                        bmp_plugin_flow_iface_base_init,   /* base_init */
                        NULL,   /* base_finalize */
                        NULL,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        0,
                        0,      /* n_preallocs */
                        NULL    /* instance_init */
                };
                type = g_type_register_static (G_TYPE_INTERFACE, "BmpPluginFlowInterface", &info, 0);
        }
        return type;
}

BmpPlaybackHistoryItem*
bmp_plugin_flow_item_prev (BmpPluginFlowInterface *self, const BmpListType *list, gint _idx)
{
	g_return_val_if_fail (BMP_IS_PLUGIN_FLOW_IFACE(self), NULL);

        return BMP_PLUGIN_FLOW_IFACE_GET_CLASS (self)->item_prev (self, list, _idx);
}

BmpPlaybackHistoryItem*
bmp_plugin_flow_item_next (BmpPluginFlowInterface *self, const BmpListType *list, gint _idx)
{
	g_return_val_if_fail (BMP_IS_PLUGIN_FLOW_IFACE(self), NULL);

        return BMP_PLUGIN_FLOW_IFACE_GET_CLASS (self)->item_next (self, list, _idx);
}


/* Transport Plugin IFace */
static void
bmp_plugin_transport_iface_base_init (gpointer g_class)
{
        static gboolean initialized = FALSE;

        if (!initialized) {
                /* create interface signals here. */
                initialized = TRUE;
        }
}

GType
bmp_plugin_transport_iface_get_type (void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (BmpPluginTransportInterfaceClass),
                        bmp_plugin_transport_iface_base_init,   /* base_init */
                        NULL,   /* base_finalize */
                        NULL,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        0,
                        0,      /* n_preallocs */
                        NULL    /* instance_init */
                };
                type = g_type_register_static (G_TYPE_INTERFACE, "BmpPluginTransportInterface", &info, 0);
        }
        return type;
}

gboolean
bmp_plugin_transport_can_process (BmpPluginTransportInterface *self, const gchar *uri) 
{
	g_return_val_if_fail (BMP_IS_PLUGIN_TRANSPORT_IFACE(self), FALSE);

        return BMP_PLUGIN_TRANSPORT_IFACE_GET_CLASS (self)->can_process (self, uri); 
}

gboolean
bmp_plugin_transport_handle_set (BmpPluginTransportInterface *self, VFSHandle *_handle)
{
	g_return_val_if_fail (BMP_IS_PLUGIN_TRANSPORT_IFACE(self), FALSE);

        return BMP_PLUGIN_TRANSPORT_IFACE_GET_CLASS (self)->handle_set (self, _handle); 
}

VFSHandle*
bmp_plugin_transport_handle_get (BmpPluginTransportInterface *self)
{
	g_return_val_if_fail (BMP_IS_PLUGIN_TRANSPORT_IFACE(self), NULL);

        return BMP_PLUGIN_TRANSPORT_IFACE_GET_CLASS (self)->handle_get (self); 
}

gboolean
bmp_plugin_transport_handle_remove (BmpPluginTransportInterface *self)
{
	g_return_val_if_fail (BMP_IS_PLUGIN_TRANSPORT_IFACE(self), FALSE);

	return BMP_PLUGIN_TRANSPORT_IFACE_GET_CLASS (self)->handle_remove (self); 
}

gboolean
bmp_plugin_transport_handle_read   (BmpPluginTransportInterface *self)
{
	g_return_val_if_fail (BMP_IS_PLUGIN_TRANSPORT_IFACE(self), FALSE);

	return BMP_PLUGIN_TRANSPORT_IFACE_GET_CLASS (self)->handle_read (self); 
}

#if 0
gboolean
bmp_plugin_transport_handle_write  (BmpPluginTransportInterface *self, HandleFreeFunc func)
{
	return BMP_PLUGIN_TRANSPORT_IFACE_GET_CLASS (self)->handle_write (self); 
}
#endif

/* Container Plugin IFace */
static void
bmp_plugin_container_iface_base_init (gpointer g_class)
{
        static gboolean initialized = FALSE;

        if (!initialized) {
                /* create interface signals here. */
                initialized = TRUE;
        }
}

GType
bmp_plugin_container_iface_get_type (void)
{
        static GType type = 0;
        if (type == 0) {
                static const GTypeInfo info = {
                        sizeof (BmpPluginContainerInterfaceClass),
                        bmp_plugin_container_iface_base_init,   /* base_init */
                        NULL,   /* base_finalize */
                        NULL,   /* class_init */
                        NULL,   /* class_finalize */
                        NULL,   /* class_data */
                        0,
                        0,      /* n_preallocs */
                        NULL    /* instance_init */
                };
                type = g_type_register_static (G_TYPE_INTERFACE, "BmpPluginContainerInterface", &info, 0);
        }
        return type;
}

gboolean
bmp_plugin_container_can_process (BmpPluginContainerInterface *self, const gchar *uri) 
{
	g_return_val_if_fail (BMP_IS_PLUGIN_CONTAINER_IFACE(self), FALSE);

        return BMP_PLUGIN_CONTAINER_IFACE_GET_CLASS (self)->can_process (self, uri); 
}

gboolean
bmp_plugin_container_handle_set (BmpPluginContainerInterface *self, VFSHandle *_handle)
{
	g_return_val_if_fail (BMP_IS_PLUGIN_CONTAINER_IFACE(self), FALSE);

        return BMP_PLUGIN_CONTAINER_IFACE_GET_CLASS (self)->handle_set (self, _handle); 
}

VFSHandle*
bmp_plugin_container_handle_get (BmpPluginContainerInterface *self)
{
	g_return_val_if_fail (BMP_IS_PLUGIN_CONTAINER_IFACE(self), NULL);

        return BMP_PLUGIN_CONTAINER_IFACE_GET_CLASS (self)->handle_get (self); 
}

gboolean
bmp_plugin_container_handle_remove (BmpPluginContainerInterface *self)
{
	g_return_val_if_fail (BMP_IS_PLUGIN_CONTAINER_IFACE(self), FALSE);

	return BMP_PLUGIN_CONTAINER_IFACE_GET_CLASS (self)->handle_remove (self); 
}

gboolean
bmp_plugin_container_handle_read   (BmpPluginContainerInterface *self)
{
	g_return_val_if_fail (BMP_IS_PLUGIN_CONTAINER_IFACE(self), FALSE);

	return BMP_PLUGIN_CONTAINER_IFACE_GET_CLASS (self)->handle_read (self); 
}

gboolean
bmp_plugin_container_handle_write  (BmpPluginContainerInterface *self)
{
	if (BMP_PLUGIN_CONTAINER_IFACE_GET_CLASS (self)->handle_write != NULL) 
	  return BMP_PLUGIN_CONTAINER_IFACE_GET_CLASS (self)->handle_write (self); 
	else
	  return FALSE;
}

const gchar**
bmp_plugin_container_get_export_data  (BmpPluginContainerInterface *self)
{
	return BMP_PLUGIN_CONTAINER_IFACE_GET_CLASS (self)->get_export_data (self); 
}
