#include <stdlib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glade/glade.h>
#include <string.h>

#include "main.h"
#include "configuration.h"
#include "system_control.h"
#include "resource_manager.h"

#include "bmpx/util.h"
#include "bmpx/uri.h"
#include "bmpx/playlist.h"

#include "glade.h"
#include "g_object_get.h"


#include "ui.h"
#include "ui_playlist.h"
#include "ui_prefswin.h"
#include "ui_about_dialog.h"

#include "bmp_window.h"
#include "bmp_slider.h"
#include "chroma_list.h"

#include "plugin_priv.h"

#include "embedded-images/media-pixbufs.h"

static GtkWidget *fi_dialog = NULL;

void
bmp_ui_callback_about (GtkAction *widget, gpointer data)
{
	bmp_about_dialog_show ();
}

void
bmp_ui_callback_preferences (GtkAction *widget, gpointer data)
{
    BmpWindowPreferences *window_preferences;

    window_preferences = RM_GET_ITEM("windows", "window-preferences", NOLOCK);

    bmp_preferences_window_show (window_preferences);
}

void
bmp_ui_callback_quit (GtkAction *widget, gpointer data)
{
	bmp_system_control_quit (bmp_system_control);
}

void
bmp_ui_callback_clear_playback_history (GtkAction *action, gpointer data)
{
    bmp_system_control_clear_playback_history (bmp_system_control, NULL);
    gtk_action_set_sensitive (gtk_action_group_get_action(bmp_actions, BMP_ACTION_CLEAR_PLAYBACK_HISTORY), FALSE);
}

void
bmp_ui_callback_iconify (GtkAction *widget, gpointer data)
{
	BmpWindowMain     *window_main;
	BmpWindowPlaylist *window_playlist;

	window_main     = RM_GET_ITEM("windows", "window-main", NOLOCK);
	window_playlist = RM_GET_ITEM("windows", "window-playlist", NOLOCK);

	gtk_window_iconify (GTK_WINDOW(window_main->window));
	gtk_window_iconify (GTK_WINDOW(window_playlist->window));
}

void
bmp_ui_callback_prev(GtkAction *widget, gpointer data)
{
	bmp_system_control_play_prev (bmp_system_control, NULL);
}

void
bmp_ui_callback_play(GtkAction *widget, gpointer data)
{
	bmp_system_control_play_current (bmp_system_control);
}

void
bmp_ui_callback_pause(GtkAction *widget, gpointer data)
{
	bmp_system_control_play_pause (bmp_system_control);
}

void
bmp_ui_callback_stop(GtkAction *widget, gpointer data)
{
	bmp_system_control_play_stop (bmp_system_control, NULL);
}

void
bmp_ui_callback_next(GtkAction *widget, gpointer data)
{
	bmp_system_control_play_next (bmp_system_control, NULL);
}

void
bmp_ui_callback_toggle_shuffle(GtkToggleAction *action, gpointer data)
{
  if (gtk_toggle_action_get_active(action))
    {
      bmp_system_control_shuffle_set (bmp_system_control, TRUE, NULL);
    }
  else
    {
      bmp_system_control_shuffle_set (bmp_system_control, FALSE, NULL);
    }
}

void
bmp_ui_callback_toggle_repeat(GtkToggleAction *action, gpointer data)
{

 if (gtk_toggle_action_get_active(action))
   {
      bmp_system_control_repeat_set (bmp_system_control, TRUE, NULL);
   }
  else
   {
      bmp_system_control_repeat_set (bmp_system_control, FALSE, NULL);
   }

}

void
bmp_ui_callback_toggle_pl(GtkToggleAction *action, gpointer data) 
{

    BmpWindowPlaylist *window_playlist;

    window_playlist = RM_GET_ITEM("windows", "window-playlist", NOLOCK);

    if (!gtk_toggle_action_get_active(action))
      {
	gtk_widget_hide (window_playlist->window);
      }
    else
      {
	gtk_widget_show_all (window_playlist->window);
	gtk_window_present (GTK_WINDOW(window_playlist->window));
	bmp_window_playlist_resize_widgets(window_playlist);
      }
}

void
bmp_ui_callback_seek_moved(GtkWidget *widget, gpointer data)
{
    BmpWindowMain *window_main;

    gint position = bmp_slider_get_position(BMP_SLIDER(widget));

    window_main = RM_GET_ITEM("windows", "window-main", NOLOCK);

    bmp_system_control_seek (bmp_system_control, position);
    bmp_window_main_set_seek_request (window_main, position);
}

void
bmp_ui_callback_volume_moved(GtkWidget *widget, gpointer data)
{
    gint volume;

    volume = bmp_slider_get_position(BMP_SLIDER(widget));
    bmp_system_control_volume_set (bmp_system_control, volume, NULL);
}

void
bmp_ui_callback_balance_moved(GtkWidget *widget, gpointer data)
{
#if 0
    gint position = bmp_slider_get_position(BMP_SLIDER(widget));
#endif
}

static gchar*
new_tracklist (BmpPlaylist *self)
{
    BmpWindowPlaylist *window_playlist;
    BmpListType *list_t;
    gchar *aux;
    gboolean match = TRUE;
    gint ctr = 0;
    gint list;

    window_playlist =
	RM_GET_ITEM("windows", "window-playlist", NOLOCK);

    while (match) {

	ctr++;
	aux = g_strdup_printf("New Playlist #%d", ctr);
	list_t = bmp_playlist_tracklist_get (self, aux);
	g_free (aux);

	if (!list_t)
	    match = FALSE;
    }

    aux = g_strdup_printf("New Playlist #%d", ctr);
    list = bmp_playlist_tracklist_new (self, aux);

    if (list != -1) {
	return aux;
    } else {
	return NULL;
    }
}

void
bmp_ui_callback_tracklist_new (GtkAction *action, gpointer data)
{
    BmpWindowPlaylist *window_playlist;
    gchar *listname;

    window_playlist =
	RM_GET_ITEM("windows", "window-playlist", NOLOCK);
    listname =
	new_tracklist (bmp_playlist);

    if (listname) {
	g_free (listname);
	chroma_list_emit_changed (CHROMA_LIST(chroma_widgets[BMP_CHROMA_LIST_PLAYLISTS]->widget));
	bmp_window_playlist_resize_widgets (window_playlist);
    }
}

/* FIXME: Make system control methods */
void
bmp_ui_callback_tracklist_remove (GtkAction *action, gpointer data)
{
    bmp_system_control_tracklist_remove (bmp_system_control, g_object_get_int (bmp_system_control, "current_tracklist"), NULL);
}

void
bmp_ui_callback_tracklists_remove_all (GtkAction *action, gpointer data)
{
    BmpWindowPlaylist *window_playlist;

    window_playlist =
	RM_GET_ITEM("windows", "window-playlist", NOLOCK);

    bmp_playlist_tracklists_remove_all (bmp_playlist);
}
/* </FIXME> */

static void
rename_tracklist (gchar *suggestion)
{
    /* FIXME: This isn't the best place for it */
    /* FIXME: The whole code is a little jack wacky */
    BmpWindowPlaylist *window_playlist;
    GladeXML *xml;
    GtkWidget *dialog;
    GtkWidget *entry;
    gint response, _idx;
    gchar *name_new = NULL, *title;

    window_playlist = RM_GET_ITEM("windows", "window-playlist", NOLOCK);

    xml = glade_xml_new_or_die(Q_("Dialogs"),  DATA_DIR "/glade/dialog_rename_tracklist.glade",
			       NULL,
                               NULL);
    dialog =
	glade_xml_get_widget (xml, "dialog_rename_tracklist");
    entry =
	glade_xml_get_widget (xml, "dialog_rename_tracklist_entry");

    if (suggestion != NULL)
	    gtk_entry_set_text (GTK_ENTRY(entry), suggestion);
    else
	    gtk_entry_set_text (GTK_ENTRY(entry), bmp_playlist_get_nth_tracklist_name (bmp_playlist, g_object_get_int (bmp_system_control, "current_tracklist")));

    gtk_editable_select_region (GTK_EDITABLE(entry), 0, -1);

    _idx =
	bmp_playlist_get_tracklist_by_name (bmp_playlist,  bmp_playlist_get_nth_tracklist_name (bmp_playlist, g_object_get_int (bmp_system_control, "current_tracklist")));

    gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW(window_playlist->window));

    repeat_rename:

    response = gtk_dialog_run (GTK_DIALOG(dialog));

    switch (response)
    {
	case GTK_RESPONSE_OK:
	  {
		name_new = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
		if (! name_new)
		{
			gtk_widget_destroy (dialog);
			return;
		}

		if (strlen(name_new) == 0)
		{
			gtk_widget_destroy (dialog);
			return;
		}
		break;
	  }
	case GTK_RESPONSE_CANCEL:
          {
    		gtk_widget_destroy (dialog);
	    	return;
          }
        default: break;
    }
    gtk_widget_hide (dialog);

    if (bmp_playlist_tracklist_rename (bmp_playlist, _idx, name_new) == -1) goto repeat_rename;
    gtk_widget_destroy (dialog);

    chroma_list_select_name (CHROMA_LIST(chroma_widgets[BMP_CHROMA_LIST_PLAYLISTS]->widget), name_new);

    title = bmp_playlist_get_current_title (bmp_playlist);
    bmp_ui_set_trayicon_text (bmp_ui, title);
    g_free (title);
}

/* FIXME: Make system control method */
void
bmp_ui_callback_tracklist_rename (GtkAction *action, gpointer data)
{
    rename_tracklist(NULL);
}

void
bmp_ui_callback_set_playlist_filename (GtkAction *action, gpointer data)
{
}

static gboolean
audio_files_filter (const GtkFileFilterInfo *filter_info,
                        gpointer                 data)
{
	if (is_audio_file(filter_info->display_name)) return TRUE;

	return FALSE;
}

static void
_filebrowser_play (GtkWidget *widget, GtkFileChooser *chooser)
{
    BmpWindowPlaylist *window_playlist;
    GSList *filenames;
    gchar **uri_list;
    gint new_ctr = 0;

    window_playlist =
	RM_GET_ITEM("windows", "window-playlist", NOLOCK);
    filenames =
	gtk_file_chooser_get_uris (GTK_FILE_CHOOSER(chooser));

    bmp_configuration_key_set_string (bmp_configuration, "bmp", "file-chooser-path", gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER(chooser)));

    if (filenames)
    {
	bmp_playlist_tracklist_remove_rows_all (bmp_playlist, g_object_get_int (bmp_system_control, "current_tracklist"));
	uri_list = gslist_to_strv (filenames);
	bmp_system_control_add_uri_list (bmp_system_control, uri_list, -1, -1, &new_ctr, NULL);
	gtk_file_chooser_unselect_all (GTK_FILE_CHOOSER(chooser));
	bmp_system_control_play_list_track (bmp_system_control, g_object_get_int(bmp_system_control, "curren_tracklist"), 0);
    }
}
static void
_filebrowser_add (GtkWidget *widget, GtkFileChooser *chooser)
{
    BmpWindowPlaylist *window_playlist;
    GSList *filenames;
    gchar **uri_list;
    gint new_ctr = 0;

    window_playlist =
	RM_GET_ITEM("windows", "window-playlist", NOLOCK);
    filenames =
	gtk_file_chooser_get_uris (GTK_FILE_CHOOSER(chooser));

    bmp_configuration_key_set_string (bmp_configuration, "bmp", "file-chooser-path", gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER(chooser)));

    if (filenames)
      {
	uri_list = gslist_to_strv (filenames);
	bmp_system_control_add_uri_list (bmp_system_control, uri_list, -1, -1, &new_ctr, NULL);
	gtk_file_chooser_unselect_all (GTK_FILE_CHOOSER(chooser));
	/* FIXME: Free GSlist */
      }
}

static void
_filebrowser_do_hide_open (GtkWidget *w1, GtkWidget *w2)
{
    if (bmp_configuration_key_fetch_boolean(bmp_configuration,"bmp","file-chooser-close-on-open"))
	gtk_widget_hide (w2);
}

static void
_filebrowser_do_hide_add (GtkWidget *w1, GtkWidget *w2)
{
    if (bmp_configuration_key_fetch_boolean(bmp_configuration,"bmp","file-chooser-close-on-add"))
	gtk_widget_hide (w2);
}

static void
_filebrowser_check_hide_open (GtkToggleButton *button, gpointer user_data)
{
    bmp_configuration_key_set_boolean(bmp_configuration,"bmp","file-chooser-close-on-open",  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)));
}
static void
_filebrowser_check_hide_add (GtkToggleButton *button, gpointer user_data)
{
    bmp_configuration_key_set_boolean(bmp_configuration,"bmp","file-chooser-close-on-add",  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)));
}

static void
_filebrowser_save_path (GtkButton *button, GtkFileChooser *chooser) {
    bmp_configuration_key_set_string (bmp_configuration, "bmp", "file-chooser-path", gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER(chooser)));
}

void
run_filebrowser(gboolean play_button)
{
    static GladeXML *xml = NULL;
    static GtkWidget *dialog = NULL;
    static GtkWidget *chooser = NULL;

    static GtkWidget *button_add, *button_close;
    static GtkWidget *button_select_all, *button_deselect_all;
    static GtkWidget *toggle;

    static GtkFileFilter *file_filter, *audio_filter;

    static gulong handlerid, handlerid_check, handlerid_do;
    static gulong handlerid_activate, handlerid_do_activate;

    if (!xml)
      {

        GtkWidget *alignment;

        xml =
	    glade_xml_new_or_die(Q_("Add/Open Files dialog"),
                                   DATA_DIR "/glade/dialog_add_files.glade",
                                   NULL, NULL);
        glade_xml_signal_autoconnect(xml);

        dialog =
	    glade_xml_get_widget(xml, "add_files_dialog");

        chooser =
	    gtk_file_chooser_widget_new(GTK_FILE_CHOOSER_ACTION_OPEN);
        gtk_file_chooser_set_select_multiple(GTK_FILE_CHOOSER(chooser), TRUE);

	/* Set Path */
	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(chooser), bmp_configuration_key_fetch (bmp_configuration, "bmp", "file-chooser-path"));

	/* Add File Filters */
	file_filter = gtk_file_filter_new();
	gtk_file_filter_set_name (file_filter, "All Files");
	gtk_file_filter_add_pattern(file_filter, "*");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), file_filter);

	audio_filter = gtk_file_filter_new();
	gtk_file_filter_set_name (audio_filter, "Audio Files");
        gtk_file_filter_add_custom (audio_filter, GTK_FILE_FILTER_DISPLAY_NAME, audio_files_filter, NULL, NULL);
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), audio_filter);

	file_filter = gtk_file_filter_new();
	gtk_file_filter_set_name (file_filter, "FLAC");
	gtk_file_filter_add_pattern(file_filter, "*.[fF][lL][aA][cC]");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), file_filter);

	file_filter = gtk_file_filter_new();
	gtk_file_filter_set_name (file_filter, "M3U/PLS");
	gtk_file_filter_add_pattern(file_filter, "*.[pP][lL][sS]");
	gtk_file_filter_add_pattern(file_filter, "*.[mM]3[uU]");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), file_filter);

#if 0
	file_filter = gtk_file_filter_new();
	gtk_file_filter_set_name (file_filter, "ZIP Archives");
	gtk_file_filter_add_pattern(file_filter, "*.[zZ][iI][pP]");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), file_filter);
#endif

	file_filter = gtk_file_filter_new();
	gtk_file_filter_set_name (file_filter, "M4A");
	gtk_file_filter_add_pattern(file_filter, "*.[mM]4[aA]");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), file_filter);

	file_filter = gtk_file_filter_new();
	gtk_file_filter_set_name (file_filter, "MP3 Audio");
	gtk_file_filter_add_pattern(file_filter, "*.[mM][pP]3");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), file_filter);

	file_filter = gtk_file_filter_new();
	gtk_file_filter_set_name (file_filter, "MPC");
	gtk_file_filter_add_pattern(file_filter, "*.[mM][pP][cC]");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), file_filter);

	file_filter = gtk_file_filter_new();
	gtk_file_filter_set_name (file_filter, "Ogg Vorbis");
	gtk_file_filter_add_pattern(file_filter, "*.[oO][gG][gG]");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), file_filter);

	file_filter = gtk_file_filter_new();
	gtk_file_filter_set_name (file_filter, "WMA");
	gtk_file_filter_add_pattern(file_filter, "*.[wW][mM][aA]");
	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(chooser), file_filter);

	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER(chooser), audio_filter);


        alignment = glade_xml_get_widget(xml, "alignment2");
        gtk_container_add(GTK_CONTAINER(alignment), chooser);

        toggle =
	    glade_xml_get_widget(xml, "close_on_action");
        button_select_all =
	    glade_xml_get_widget(xml, "select_all");
        button_deselect_all =
	    glade_xml_get_widget(xml, "deselect_all");
        button_add =
	    glade_xml_get_widget(xml, "action");
	button_close =
	    glade_xml_get_widget(xml, "close");

        g_signal_connect_swapped(button_select_all, "clicked",
                                 G_CALLBACK(gtk_file_chooser_select_all),
                                 chooser);

        g_signal_connect_swapped(button_deselect_all, "clicked",
                                 G_CALLBACK(gtk_file_chooser_unselect_all),
                                 chooser);

        g_signal_connect(button_close, "clicked",
                                 G_CALLBACK(_filebrowser_save_path),
                                 chooser);

        g_signal_connect_swapped(button_close, "clicked",
                                 G_CALLBACK(gtk_widget_hide),
                                 dialog);


#if 0
        g_signal_connect(dialog, "key_press_event",
                         G_CALLBACK(filebrowser_on_keypress),
                         NULL);
#endif

        gtk_widget_show_all(dialog);
      } /* !xml */
    else
      {
        g_signal_handler_disconnect(button_add, handlerid);
        g_signal_handler_disconnect(toggle, handlerid_check);
        g_signal_handler_disconnect(chooser, handlerid_activate);
        g_signal_handler_disconnect(button_add, handlerid_do);
        g_signal_handler_disconnect(chooser, handlerid_do_activate);

	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(chooser), bmp_configuration_key_fetch (bmp_configuration, "bmp", "file-chooser-path"));
      }

    if (play_button)
      {
        gtk_window_set_title(GTK_WINDOW(dialog), Q_("Open Files"));
        gtk_button_set_label(GTK_BUTTON(button_add), GTK_STOCK_OPEN);
        gtk_button_set_label(GTK_BUTTON(toggle), Q_("Close dialog on Open"));

        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle),
	    bmp_configuration_key_fetch_boolean(bmp_configuration, "bmp", "file-chooser-close-on-open"));

        handlerid_check =
	    g_signal_connect(toggle,
			"toggled",
			G_CALLBACK(_filebrowser_check_hide_open),
			NULL);

        handlerid =
	    g_signal_connect(button_add,
			"clicked",
			G_CALLBACK(_filebrowser_play),
			chooser);

        handlerid_do =
	    g_signal_connect_after(button_add,
			"clicked",
			G_CALLBACK(_filebrowser_do_hide_open),
			dialog);

        handlerid_activate =
	    g_signal_connect(chooser,
			"file-activated",
			G_CALLBACK(_filebrowser_play),
			chooser);

        handlerid_do_activate =
	    g_signal_connect_after(chooser,
			"file-activated",
			G_CALLBACK(_filebrowser_do_hide_open),
			dialog);

      }
    else
      {
        gtk_window_set_title(GTK_WINDOW(dialog), Q_("Add Files"));
        gtk_button_set_label(GTK_BUTTON(button_add), GTK_STOCK_ADD);
        gtk_button_set_label(GTK_BUTTON(toggle), Q_("Close dialog on Add"));

        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle),
	    bmp_configuration_key_fetch_boolean(bmp_configuration, "bmp", "file-chooser-close-on-add"));

        handlerid_check =
	    g_signal_connect(toggle,
			"toggled",
			G_CALLBACK(_filebrowser_check_hide_add),
			NULL);

        handlerid =
	    g_signal_connect(button_add,
			"clicked",
			G_CALLBACK(_filebrowser_add),
			chooser);

        handlerid_do =
	    g_signal_connect_after(button_add,
			"clicked",
			G_CALLBACK(_filebrowser_do_hide_add),
			dialog);

        handlerid_activate =
	    g_signal_connect(chooser,
			"file-activated",
			G_CALLBACK(_filebrowser_add),
			chooser);

        handlerid_do_activate =
	    g_signal_connect_after(chooser,
			"file-activated",
			G_CALLBACK(_filebrowser_do_hide_add),
			dialog);

      }

    gtk_window_present(GTK_WINDOW(dialog));
}

void
bmp_ui_callback_add_url (GtkAction *action, gpointer data)
{
    const gchar *glade_file = DATA_DIR "/glade/dialog_add_url.glade";
    static GladeXML *xml = NULL;
    static GtkWidget *add_url_dialog;
    gint response, new_ctr = 0;

    if (!xml)
      {
        xml =
	    glade_xml_new_or_die(Q_("Dialogs"), glade_file, NULL, NULL);
        add_url_dialog =
		glade_xml_get_widget_warn (xml, "dialog_add_url");
      }

    gtk_window_set_position (GTK_WINDOW(add_url_dialog), GTK_WIN_POS_CENTER);
    gtk_widget_set_size_request (GTK_WIDGET(add_url_dialog), 400, -1);

    response =
	gtk_dialog_run (GTK_DIALOG(add_url_dialog));

    switch (response)
      {

	case GTK_RESPONSE_OK:
          {

	    const gchar *uri_list[2];

	    uri_list[0] = gtk_entry_get_text (GTK_ENTRY(glade_xml_get_widget_warn (xml, "entry_url")));
	    uri_list[1] = NULL;

	    bmp_system_control_add_uri_list (bmp_system_control, (gchar**)uri_list, -1, -1, &new_ctr, NULL);

	  }
        break;

	default: break;

      }

    gtk_widget_hide (add_url_dialog);
}

void
bmp_ui_callback_add_files (GtkAction *action, gpointer data)
{
    run_filebrowser (FALSE);
}

void
bmp_ui_callback_open_files (GtkAction *action, gpointer data)
{
    run_filebrowser (TRUE);
}

#ifdef HAVE_XINE
void
bmp_ui_callback_add_cd (GtkAction *action, gpointer data)
{
    BmpWindowPlaylist *window_playlist;
    gchar **mrls, **strv, *disc_id;
    gchar **fragments;
    gchar  *full_uri;
    gint n, n_mrls, new_ctr = 0;

    mrls = bmp_play_get_cd (bmp_play, &n_mrls, &disc_id);
    if ( (! mrls) || (!n_mrls)) return;

    strv = g_new (gchar *, n_mrls+1);
    for (n = 0; n < n_mrls; n++)
      {
	fragments = 
	    g_strsplit (mrls[n],"/",-1);	
	full_uri = 
	    g_strconcat (fragments[0],"//",disc_id,"/",fragments[1],NULL);
	strv[n] = g_strdup (full_uri); 
	g_free (full_uri);
	g_strfreev (fragments);
      }
    strv[n] = NULL;

    bmp_system_control_add_uri_list (bmp_system_control, strv, -1, -1, &new_ctr, NULL);

    window_playlist = RM_GET_ITEM("windows", "window-playlist", NOLOCK);
}

#else
void
bmp_ui_callback_add_cd (GtkAction *action, gpointer data)
{
}
#endif

void
bmp_ui_callback_del_files (GtkAction *action, gpointer data)
{
    BmpWindowPlaylist *window_playlist;
    ChromaList *list;

    window_playlist = RM_GET_ITEM("windows", "window-playlist", NOLOCK);
    list = CHROMA_LIST(chroma_widgets[BMP_CHROMA_LIST_TRACKLIST]->widget);

    bmp_playlist_tracklist_remove_selected (bmp_playlist, g_object_get_int (bmp_system_control, "current_tracklist"));
}

void
bmp_ui_callback_crop_files (GtkAction *action, gpointer data)
{
    BmpWindowPlaylist *window_playlist;
    ChromaList *list;

    window_playlist = RM_GET_ITEM("windows", "window-playlist", NOLOCK);
    list = CHROMA_LIST(chroma_widgets[BMP_CHROMA_LIST_TRACKLIST]->widget);

    bmp_playlist_tracklist_remove_unselected (bmp_playlist, g_object_get_int (bmp_system_control, "current_tracklist"));
}

void
bmp_ui_callback_del_files_all (GtkAction *action, gpointer data)
{
    BmpWindowPlaylist *window_playlist;
    ChromaList *list;

    window_playlist = RM_GET_ITEM("windows", "window-playlist", NOLOCK);
    list = CHROMA_LIST(chroma_widgets[BMP_CHROMA_LIST_TRACKLIST]->widget);

    bmp_playlist_tracklist_remove_rows_all (bmp_playlist, g_object_get_int (bmp_system_control, "current_tracklist"));
    bmp_system_control_clear_playback_history (bmp_system_control, NULL);
    gtk_action_set_sensitive (gtk_action_group_get_action(bmp_actions, BMP_ACTION_CLEAR_PLAYBACK_HISTORY), FALSE);
}

/* Well, WTF you will be asking here. All the above functions operate on the list itself, while this operates with ChromaList.
   The explanation is like this: The above functions perform stuff directly related to the list, AFTER a selection has been made.
   After a selection has been made, a certain operation on the list items happens, but THIS function does something that is a pure
   UI-thing. It selects all items. Same with unselect all etc */

void
bmp_ui_callback_select_all_files (GtkAction *action, gpointer data)
{
	ChromaList *list;

	list = CHROMA_LIST(chroma_widgets[BMP_CHROMA_LIST_TRACKLIST]->widget);
	chroma_list_select_all (list);
}

void
bmp_ui_callback_unselect_all_files (GtkAction *action, gpointer data)
{
	ChromaList *list;

	list = CHROMA_LIST(chroma_widgets[BMP_CHROMA_LIST_TRACKLIST]->widget);
	chroma_list_unselect_all (list);
}

void
bmp_ui_callback_invert_selection (GtkAction *action, gpointer data)
{
	ChromaList *list;

	list = CHROMA_LIST(chroma_widgets[BMP_CHROMA_LIST_TRACKLIST]->widget);
	chroma_list_invert_selection (CHROMA_LIST(list));
}

void
bmp_ui_callback_sort_current_list (GtkAction *action, gpointer data)
{
	bmp_playlist_sort_current_list (bmp_playlist);
}

void
bmp_ui_callback_sort_current_list_artist (GtkAction *action, gpointer data)
{
	bmp_playlist_sort_current_list_by_tag (bmp_playlist, BMP_DATUM_ARTIST);
}

void
bmp_ui_callback_sort_current_list_album (GtkAction *action, gpointer data)
{
	bmp_playlist_sort_current_list_by_tag (bmp_playlist, BMP_DATUM_ALBUM);
}

void
bmp_ui_callback_sort_current_list_title (GtkAction *action, gpointer data)
{
	bmp_playlist_sort_current_list_by_tag (bmp_playlist, BMP_DATUM_TITLE);
}

void
bmp_ui_callback_sort_current_list_genre (GtkAction *action, gpointer data)
{
	bmp_playlist_sort_current_list_by_tag (bmp_playlist, BMP_DATUM_GENRE);
}

void
bmp_ui_callback_sort_current_list_track (GtkAction *action, gpointer data)
{
	bmp_playlist_sort_current_list_by_tag (bmp_playlist, BMP_DATUM_TRACK);
}

void
bmp_ui_callback_sort_current_list_location (GtkAction *action, gpointer data)
{
	bmp_playlist_sort_current_list_by_tag (bmp_playlist, BMP_DATUM_LOCATION);
}

static void
fileinfo_update (void)
{

#define ENTRY_SET_TEXT(widget, tag_id) \
	{ \
		GtkWidget *entry; \
		gchar *text; \
		\
		entry = glade_xml_get_widget (xml, widget); \
		if (item->metadata) \
		{ \
		    text = g_hash_table_lookup(item->metadata, bmp_metadata_get_id_static(tag_id)); \
		    if (text) \
			    gtk_widget_set_sensitive(entry, TRUE); \
		    else \
		    { \
			    gtk_widget_set_sensitive(entry, FALSE); \
			    text = ""; \
		    } \
		} \
		else \
		{ \
		    text = ""; \
		    gtk_widget_set_sensitive(entry, FALSE); \
		} \
		\
		if (tag_id == BMP_DATUM_LOCATION) \
	          { \
		    GURI *uri; \
		    gchar *text_uri; \
		    uri = bmp_uri_new (text); \
		    bmp_uri_unescape (uri); \
		    text_uri = bmp_uri_get_string(uri); \
		    gtk_entry_set_text (GTK_ENTRY(entry), text_uri);\
		    g_free (text_uri); \
		    bmp_uri_delete (uri);\
		  } \
		else \
		 { \
		    gtk_entry_set_text (GTK_ENTRY(entry), text); \
		 } \
	}


#define LABEL_SET_TEXT(widget, text) \
	{ \
		GtkWidget *label; \
		label = glade_xml_get_widget (xml, widget); \
		gtk_label_set_text (label, text); \
	}

#define LABEL_SET_TEXT_METADATA(widget, tag_id, append) \
	{ \
		GtkWidget *label; \
		gchar *text, *text_fq; \
		\
		label = glade_xml_get_widget (xml, widget); \
		if (item->metadata) \
		{ \
		    text = g_hash_table_lookup(item->metadata, bmp_metadata_get_id_static(tag_id)); \
		    if (text) \
			    gtk_widget_set_sensitive(label, TRUE); \
		    else \
			    gtk_widget_set_sensitive(label, FALSE); \
		} \
		else \
		{ \
		    text = ""; \
		    gtk_widget_set_sensitive(label, FALSE); \
		} \
		\
		text_fq = g_strdup_printf("%s %s", text, append); \
		gtk_label_set_text (GTK_LABEL(label), text_fq); \
		g_free (text_fq); \
	}


    const gchar *glade_file = DATA_DIR "/glade/dialog_file_info.glade";
    GtkWidget *fi_image;
    static GladeXML *xml;
    BmpListType *list;
    BmpFileItem *item;
    GList *paths;
    gint idx;

    paths = chroma_list_get_selected_paths (CHROMA_LIST(chroma_widgets[BMP_CHROMA_LIST_TRACKLIST]->widget));

    if (! paths) return;
    if (g_list_length (paths) > 1)
    {
	gtk_tree_path_free (paths->data);
	return;
    }

    idx = gtk_tree_path_get_indices(paths->data)[0];

    if (! fi_dialog)
    {
        xml = glade_xml_new_or_die(Q_("Dialogs"), glade_file, NULL, NULL);
	fi_dialog = glade_xml_get_widget(xml, "dialog_file_information");
	gtk_widget_realize (fi_dialog);
	gtk_window_set_position (GTK_WINDOW(fi_dialog), GTK_WIN_POS_CENTER_ALWAYS);
	g_signal_connect_swapped (G_OBJECT(glade_xml_get_widget(xml,"fi_ok")), "clicked", G_CALLBACK(gtk_widget_hide), fi_dialog);
	fi_image = glade_xml_get_widget(xml, "fi_image");
	gtk_image_set_from_file (GTK_IMAGE(fi_image), DATA_DIR "/images/audio.png");
    }

    list = BMP_PLAYLIST_GET_CURRENT(bmp_playlist);
    item = g_list_nth (list->items, idx)->data;

    ENTRY_SET_TEXT("entry_file",    BMP_DATUM_LOCATION);
    ENTRY_SET_TEXT("entry_artist",  BMP_DATUM_ARTIST)
    ENTRY_SET_TEXT("entry_album",   BMP_DATUM_ALBUM)
    ENTRY_SET_TEXT("entry_title",   BMP_DATUM_TITLE)
    ENTRY_SET_TEXT("entry_genre",   BMP_DATUM_GENRE)
    ENTRY_SET_TEXT("entry_date",    BMP_DATUM_DATE)
    ENTRY_SET_TEXT("entry_comment", BMP_DATUM_COMMENT)
    ENTRY_SET_TEXT("entry_track",   BMP_DATUM_TRACK)

    LABEL_SET_TEXT_METADATA("label_bitrate", BMP_DATUM_BITRATE, "kbit/s")

    gtk_tree_path_free (paths->data);
}

void
bmp_ui_callback_fileinfo_update (void)
{
    if (! fi_dialog) return;
    if (! GTK_WIDGET_VISIBLE(fi_dialog)) return;
    fileinfo_update ();
    gtk_widget_set_size_request (GTK_WIDGET(fi_dialog), 600, -1);
}

void
bmp_ui_callback_fileinfo (GtkAction *action, gpointer data)
{
    fileinfo_update ();
    gtk_widget_set_size_request (GTK_WIDGET(fi_dialog), 600, -1);

    gtk_window_present (GTK_WINDOW(fi_dialog));
}


/* Library search dialog */
#define LIB_VIEW_SEARCH_N_COLS 4

static GtkTreeViewColumn* library_column_search[LIB_VIEW_SEARCH_N_COLS];
static GtkCellRenderer*   library_renderer_search[LIB_VIEW_SEARCH_N_COLS];

static GtkTreeViewColumn* library_column_browse[2];
static GtkCellRenderer*   library_renderer_browse[3];

static gchar* column_titles[] = {
	N_("Artist"),
	N_("Album"),
	N_("Title"),
	N_("Track"),
};

static gint column_tag_ids[] = {
	BMP_DATUM_ARTIST,
	BMP_DATUM_ALBUM,
	BMP_DATUM_TITLE,
	BMP_DATUM_TRACK
};

static void
on_button_enqueue_clicked (GtkButton *button, GladeXML *xml)
{
    GtkNotebook *library_notebook;
    GtkWidget *tree_view;
    GtkTreeModel *store;
    GtkTreeSelection *selection;
    GList *paths, *uris = NULL;
    GtkTreeIter iter;
    gchar **uri_list;
    gchar *uri;
    gint new_ctr = 0;

    library_notebook =
        GTK_NOTEBOOK (glade_xml_get_widget_warn (xml, "library_notebook"));

    if (gtk_notebook_get_current_page(library_notebook) == 0)
      {
        tree_view =
	    glade_xml_get_widget_warn (xml, "tree_view_search");
	store =
	    gtk_tree_view_get_model (GTK_TREE_VIEW(tree_view));
	selection =
	    gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));
	paths =
	    gtk_tree_selection_get_selected_rows (selection, NULL);

        if (! paths) {
	    gtk_tree_selection_select_all (selection);
	    paths =
		gtk_tree_selection_get_selected_rows (selection, NULL);
	}

	while (paths)
	{
	    gtk_tree_model_get_iter (store, &iter, paths->data);
	    gtk_tree_model_get (store, &iter, LIB_VIEW_SEARCH_N_COLS, &uri, -1);

	    uris = g_list_append (uris, uri);
	    paths = g_list_next (paths);
	}

	uri_list = glist_to_strv (uris);

	bmp_system_control_add_uri_list (bmp_system_control,
					    uri_list,
					    -1,
					    -1,
					    &new_ctr,
					    NULL);

	g_list_foreach (paths, (GFunc)gtk_tree_path_free, NULL);
	g_list_free (paths);

	gtk_tree_selection_unselect_all (selection);
      }
    else
      {

        tree_view =
	    glade_xml_get_widget_warn (xml, "tree_view_browse");
	store =
	    gtk_tree_view_get_model (GTK_TREE_VIEW(tree_view));
	selection =
	    gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));
	paths =
	    gtk_tree_selection_get_selected_rows (selection, NULL);

	for ( ; paths; paths = paths->next)
          {
	    gint depth;

	    depth = gtk_tree_path_get_depth (paths->data);

	    switch (depth)
             {
               case 1:
                {
		    /* Enqueue artist */

                  GtkTreeIter iter;
		  GList *iter_list;
		  gchar *artist;
		  gchar **uri_list;
		  gint n = 0, new_ctr = 0;
		  bmp_query_t *query;

		  gtk_tree_model_get_iter (GTK_TREE_MODEL(store), &iter, paths->data);
		  gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, 1, &artist, -1);

		  query = bmp_metadata_cache_query_construct (bmp_metadata_cache, TRUE, BMP_DATUM_ALBUM,
							      1,
							      BMP_DATUM_ARTIST, artist, TRUE);

		  bmp_metadata_cache_query_run (bmp_metadata_cache, query);
		  g_free (artist);

		  iter_list = query->result_ufid_list;

		  uri_list = g_new (gchar *, g_list_length(iter_list)+1);

		  for ( ; iter_list; iter_list = iter_list->next)
                    {

		      uri_list[n] = (gchar*)
			bmp_metadata_cache_get_metadata_field_from_ufid (bmp_metadata_cache,
									 GPOINTER_TO_INT(iter_list->data),
									 BMP_DATUM_LOCATION);
		      n++;
                    }
		  uri_list[n] = NULL;

		  bmp_system_control_add_uri_list (bmp_system_control, uri_list, -1, -1, &new_ctr, NULL);
                } break;

               case 2:
                {
		    /* Enqueue album */

		  GtkTreeIter iter, iter_parent;
		  GList *iter_list;
		  gchar *artist, *album;
		  gchar **uri_list;
		  gint n = 0, new_ctr = 0;
		  bmp_query_t *query;

		  gtk_tree_model_get_iter (GTK_TREE_MODEL(store), &iter, paths->data);
		  gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, 1, &album, -1);

		  gtk_tree_model_iter_parent (GTK_TREE_MODEL(store), &iter_parent, &iter);
		  gtk_tree_model_get (GTK_TREE_MODEL(store), &iter_parent, 1, &artist, -1);

		  query = bmp_metadata_cache_query_construct (bmp_metadata_cache, TRUE, BMP_DATUM_TITLE,
							      2,
							      BMP_DATUM_ARTIST, artist, TRUE,
							      BMP_DATUM_ALBUM,  album,  TRUE);
		  bmp_metadata_cache_query_run (bmp_metadata_cache, query);
		  g_free (artist);
		  g_free (album);

		  iter_list = query->result_ufid_list;

		  uri_list = g_new (gchar *, g_list_length(iter_list)+1);

		  for ( ; iter_list; iter_list = iter_list->next)
                    {

		      uri_list[n] = (gchar*)
			bmp_metadata_cache_get_metadata_field_from_ufid (bmp_metadata_cache,
									 GPOINTER_TO_INT(iter_list->data),
									 BMP_DATUM_LOCATION);
		      n++;
                    }
		  uri_list[n] = NULL;

		  bmp_system_control_add_uri_list (bmp_system_control, uri_list, -1, -1, &new_ctr, NULL);

                } break;


	       case 3:
                {
		   /* Add song to UFID list */

		    gtk_tree_model_get_iter (store, &iter, paths->data);
		    gtk_tree_model_get (store, &iter, 2, &uri, -1);

		    uris = g_list_append (uris, uri);

                } break;
             }
          }
	    uri_list = glist_to_strv (uris);

	    bmp_system_control_add_uri_list (bmp_system_control,
					    uri_list,
					    -1,
					    -1,
					    &new_ctr,
					    NULL);

	    g_list_foreach (uris, (GFunc) g_free, NULL);
	    g_list_free (uris);

      }
}

static void
on_button_play_clicked (GtkButton *button, GladeXML *xml)
{
    gint tracklist;

    tracklist = g_object_get_int (bmp_system_control, "current_tracklist");

    bmp_playlist_tracklist_remove_rows_all (bmp_playlist, tracklist);
    on_button_enqueue_clicked (GTK_BUTTON (glade_xml_get_widget_warn (xml, "button_enqueue")), xml);
    bmp_system_control_play_list_track (bmp_system_control, tracklist, 0);
}

static void
on_tree_view_search_row_activated (GtkTreeView *treeview,
                                   GtkTreePath *path,
                                   GtkTreeViewColumn *column,
                                   GladeXML *xml)
{
    GtkTreeModel *store;
    GtkTreeIter iter;
    gchar *uri_list[2];
    gchar *uri;
    gint new_ctr = 0;

    store =
	gtk_tree_view_get_model (treeview);

    gtk_tree_model_get_iter (store, &iter, path);
    gtk_tree_model_get (store, &iter, LIB_VIEW_SEARCH_N_COLS, &uri, -1);

    uri_list[0] =
	    uri;
    uri_list[1] =
	    NULL;

    bmp_system_control_add_uri_list (bmp_system_control,
				uri_list,
				-1,
				-1,
				&new_ctr,
				NULL);
}

static void
on_cbox_tag_id_changed (GtkComboBox *cbox, GladeXML *xml) {

    GtkWidget *editable;

    g_return_if_fail (GTK_IS_COMBO_BOX (cbox));

    editable = glade_xml_get_widget_warn (xml, "entry_value");
    g_signal_emit_by_name (editable, "changed");
}

static void
on_cbox_sort_changed (GtkComboBox *cbox, GladeXML *xml) {

    GtkWidget *editable;

    g_return_if_fail (GTK_IS_COMBO_BOX (cbox));

    editable = glade_xml_get_widget_warn (xml, "entry_value");
    g_signal_emit_by_name (editable, "changed");
}

static void
on_cb_exact_changed (GtkToggleButton *button, GladeXML *xml) {

    GtkWidget *editable;

    g_return_if_fail (GTK_TOGGLE_BUTTON (button));

    editable = glade_xml_get_widget_warn (xml, "entry_value");
    g_signal_emit_by_name (editable, "changed");
}


static void
on_entry_value_changed (GtkEditable *editable, GladeXML *xml) {

    GtkWidget *tree_view;
    GtkListStore *list_store_old, *list_store, *list_store_completion;
    GtkEntryCompletion *entry_completion;
    GtkTreeIter iter;
    GType types[LIB_VIEW_SEARCH_N_COLS+1];
    bmp_query_t *query;
    gint      sort_tag_id;
    gint      tag_id;
    gchar    *match_value;
    gboolean  exact_match;
    gint n;

    g_return_if_fail (GTK_IS_EDITABLE(editable));

    for (n = 0; n < LIB_VIEW_SEARCH_N_COLS+1; types[n++] = G_TYPE_STRING);

    tree_view =
	glade_xml_get_widget_warn (xml, "tree_view_search");
    list_store =
	gtk_list_store_newv (LIB_VIEW_SEARCH_N_COLS+1, types);

    sort_tag_id =
	gtk_combo_box_get_active (GTK_COMBO_BOX (glade_xml_get_widget_warn (xml, "cbox_sort")));
    tag_id =
	gtk_combo_box_get_active (GTK_COMBO_BOX (glade_xml_get_widget_warn (xml, "cbox_tag_id")));
    match_value =
	gtk_editable_get_chars (editable, 0, -1);
    exact_match =
	gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (glade_xml_get_widget_warn (xml, "cb_exact")));

    query =
	bmp_metadata_cache_query_construct (bmp_metadata_cache, TRUE, sort_tag_id,
						1,
					        tag_id,
						match_value,
						exact_match);

    bmp_metadata_cache_query_run (bmp_metadata_cache, query);

    while (query->result_ufid_list) {

	GHashTable *metadata;
	gchar *value;
	gint n;

	metadata =
	    bmp_metadata_cache_get_metadata_for_ufid (bmp_metadata_cache, GPOINTER_TO_INT (query->result_ufid_list->data));

	gtk_list_store_append (list_store, &iter);

	for (n = 0; n < LIB_VIEW_SEARCH_N_COLS; n++) {
	    value =
		g_hash_table_lookup (metadata, bmp_metadata_get_id_static(column_tag_ids[n]));

	    if (value) {
		if (g_utf8_validate(value, strlen(value), NULL)) {
		    gtk_list_store_set (list_store, &iter, n, value, -1);
		} else {
		    gtk_list_store_set (list_store, &iter, n, "", -1);
		}
	    }

	}

	value =
	    g_hash_table_lookup (metadata, bmp_metadata_get_id_static(BMP_DATUM_LOCATION));
	gtk_list_store_set (list_store, &iter, LIB_VIEW_SEARCH_N_COLS, value, -1);

	query->result_ufid_list = g_list_next (query->result_ufid_list);
    }

    list_store_old =
        GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW(tree_view)));
    gtk_tree_view_set_model (GTK_TREE_VIEW(tree_view), GTK_TREE_MODEL(list_store));
    if (list_store_old)
	g_object_unref (list_store_old);
    gtk_tree_view_columns_autosize (GTK_TREE_VIEW(tree_view));

    /* Create the completion entry */
    list_store_completion =
	bmp_metadata_cache_list_tag_id (bmp_metadata_cache, tag_id);
    entry_completion =
	gtk_entry_get_completion (GTK_ENTRY(editable));
    list_store_old =
        GTK_LIST_STORE (gtk_entry_completion_get_model (entry_completion));
    gtk_entry_completion_set_model (entry_completion, GTK_TREE_MODEL (list_store_completion));
    if (list_store_old)
	g_object_unref (list_store_old);

    bmp_metadata_cache_query_dispose (bmp_metadata_cache, query);
}

static gboolean
on_library_dialog_key_press (GtkWidget *widget,
		    GdkEventKey *event,
		    gpointer user_data)
{
    g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);

    if (event->keyval == GDK_Escape)
	gtk_widget_hide (widget);

    return FALSE;
}

static gint
ufid_datum_compare_func (gconstpointer a, gconstpointer b, gpointer user_data)
{
    gint ufid_a, ufid_b, track_ai, track_bi;
    const gchar *track_a, *track_b;

    ufid_a = GPOINTER_TO_INT (a);
    ufid_b = GPOINTER_TO_INT (b);

    /* FIXME: We ignore tag_id and sort only by track currently */

    track_a = bmp_metadata_cache_get_metadata_field_from_ufid (bmp_metadata_cache, ufid_a, BMP_DATUM_TRACK);
    track_b = bmp_metadata_cache_get_metadata_field_from_ufid (bmp_metadata_cache, ufid_b, BMP_DATUM_TRACK);

    if ((!track_a) && ( track_b)) return -1;
    if ((!track_a) && (!track_b)) return  0;
    if (( track_a) && (!track_b)) return  1;

    track_ai = strtol (track_a,NULL,10);
    track_bi = strtol (track_b,NULL,10);

    return track_ai - track_bi;
}


/* FIXME: This function is rather slow */
#if 0
static void
on_browse_notebook_realize (GtkNotebook *browse_notebook, gpointer user_data)
{
    enum {
	    PB_ARTIST,
	    PB_ALBUM,
	    PB_TITLE,

	    N_PB
    };

    GtkTreeView *tree_view_browse;
    GtkWidget *label_info;
    GtkTreeStore *store;
    GdkPixbuf *pb[N_PB];
    GtkTreeIter iter, iter_child_album, iter_child_title;
    GList *list, *list_albums, *list_titles;
    gint artists = 0, albums = 0, titles = 0;

    browse_notebook =
        GTK_NOTEBOOK (glade_xml_get_widget_warn (GLADE_XML(user_data), "browse_notebook"));
    tree_view_browse =
        GTK_TREE_VIEW (glade_xml_get_widget_warn (GLADE_XML(user_data), "tree_view_browse"));
    label_info =
	glade_xml_get_widget_warn (GLADE_XML(user_data), "label_info");
    store =
	gtk_tree_store_new (3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);

    pb[PB_ARTIST] = gdk_pixbuf_new_from_inline (-1, icon_artist, FALSE, NULL);
    pb[PB_ALBUM]  = gdk_pixbuf_new_from_inline (-1, icon_album, FALSE, NULL);
    pb[PB_TITLE]  = gdk_pixbuf_new_from_inline (-1, icon_title, FALSE, NULL);

    /* Add data */
    list = bmp_metadata_cache_list_tag_id_glist (bmp_metadata_cache, BMP_DATUM_ARTIST);
    list = g_list_sort (list, (GCompareFunc) g_utf8_collate);

    for (; list; list = g_list_next(list))
      {
        gchar *artist, *album;
	const gchar *title, *location;
	gpointer ufid;

        artist = list->data;
	gtk_tree_store_append (store, &iter, NULL);
	gtk_tree_store_set (store, &iter, 0, pb[PB_ARTIST], 1, artist, -1);
	artists++;

	/* Now we append albums to the artist */
          {
	    list_albums = bmp_metadata_cache_get_albums_for_artist (bmp_metadata_cache, artist);
	    list_albums = g_list_sort (list_albums, (GCompareFunc)g_utf8_collate);

	    for (; list_albums; list_albums = g_list_next(list_albums))
              {
		 album = list_albums->data;
		    
		 if (album)
                   {
                      gtk_tree_store_append (store, &iter_child_album, &iter);
		      gtk_tree_store_set (store, &iter_child_album, 0, pb[PB_ALBUM], 1, album, -1);
		      albums++;

		      /* Now append tracks to the album */
	              list_titles = bmp_metadata_cache_get_songs_for_album (bmp_metadata_cache, album);
		      list_titles = g_list_sort_with_data (list_titles, ufid_datum_compare_func, GINT_TO_POINTER (BMP_DATUM_TRACK));

		      for (; list_titles; list_titles = g_list_next(list_titles))
                        {
		           ufid = 
				list_titles->data;
			   location =
				bmp_metadata_cache_get_metadata_field_from_ufid (bmp_metadata_cache, GPOINTER_TO_INT(ufid), BMP_DATUM_LOCATION);
			   title =
				bmp_metadata_cache_get_metadata_field_from_ufid (bmp_metadata_cache, GPOINTER_TO_INT(ufid), BMP_DATUM_TITLE);
		               
			   gtk_tree_store_append (store, &iter_child_title, &iter_child_album);
			   gtk_tree_store_set (store, &iter_child_title, 0, pb[PB_TITLE], 1, title, 2, location, -1);
			   titles++;
                        }
                   }
              }

	    gc_mark (ngc);
	    gtk_label_set_markup (GTK_LABEL(label_info), gc_collect (ngc, g_strdup_printf("<span size=\"small\">Artists: %d | Albums: %d | Titles: %d</span>", artists, albums, titles)));
	    gc_free (ngc);
	    while (gtk_events_pending()) gtk_main_iteration();
          }
      }

    g_list_foreach (list, (GFunc) g_free, NULL);
    g_free (list);

    gtk_tree_view_set_model (tree_view_browse, GTK_TREE_MODEL (store));
    gtk_notebook_set_current_page (browse_notebook, 1);
}
#endif

void
bmp_ui_callback_add_files_library (GtkAction *action, gpointer data)
{
    const gchar *glade_file = DATA_DIR "/glade/dialog_add_from_library.glade";
    static GladeXML *xml;
    static GtkWidget *dialog_window = NULL;
    GtkEntryCompletion *entry_completion;
    GtkWidget *widget_aux;
    gint n;

    if (! dialog_window)
    {
	xml =
	    glade_xml_new_or_die(Q_("Dialogs"), glade_file, NULL, NULL);
	dialog_window =
	    glade_xml_get_widget(xml, "library_dialog");
	gtk_widget_realize (dialog_window);
	gtk_window_set_position (GTK_WINDOW(dialog_window), GTK_WIN_POS_CENTER_ALWAYS);

	g_object_connect (G_OBJECT(dialog_window), "signal::key-press-event", G_CALLBACK(on_library_dialog_key_press), NULL, NULL);

	/* Setup SEARCH Tab */

   	widget_aux =
	    glade_xml_get_widget(xml, "cbox_tag_id");
	/* Remove the dummy entry */
	gtk_combo_box_remove_text (GTK_COMBO_BOX(widget_aux), 0);
	for (n = 0; n < BMP_DATUM_NONE; n++)
	    gtk_combo_box_append_text (GTK_COMBO_BOX(widget_aux), bmp_metadata_get_title(n));

   	widget_aux =
	    glade_xml_get_widget(xml, "cbox_sort");
	/* Remove the dummy entry */
	gtk_combo_box_remove_text (GTK_COMBO_BOX(widget_aux), 0);
	for (n = 0; n < BMP_DATUM_NONE; n++)
	    gtk_combo_box_append_text (GTK_COMBO_BOX(widget_aux), bmp_metadata_get_title(n));

	widget_aux =
	    glade_xml_get_widget(xml, "tree_view_search");

	gtk_tree_selection_set_mode
	    (gtk_tree_view_get_selection (GTK_TREE_VIEW(widget_aux)),
	     GTK_SELECTION_MULTIPLE);

	g_object_connect(G_OBJECT(widget_aux), "signal::row-activated", G_CALLBACK(on_tree_view_search_row_activated), xml, NULL);

	for (n = 0; n < LIB_VIEW_SEARCH_N_COLS; n++) {

	    library_column_search[n] =
		gtk_tree_view_column_new ();

	    library_renderer_search[n] =
		gtk_cell_renderer_text_new ();

	    gtk_tree_view_column_pack_start
		(library_column_search[n],
		 library_renderer_search[n],
		 FALSE);

	    gtk_tree_view_column_set_attributes
		(library_column_search[n],
		 library_renderer_search[n],
		 "text",
		  n,
		  NULL);

	    gtk_tree_view_column_set_expand
		(library_column_search[n],
		 FALSE);

	    gtk_tree_view_column_set_title
		(library_column_search[n],
		 column_titles[n]);

	    gtk_tree_view_column_set_sizing
		(library_column_search[n],
		 GTK_TREE_VIEW_COLUMN_GROW_ONLY);

	    gtk_tree_view_column_set_resizable
		(library_column_search[n],
		 TRUE);

	    gtk_tree_view_append_column
		(GTK_TREE_VIEW (widget_aux),
		 GTK_TREE_VIEW_COLUMN (library_column_search[n]));

	}

	widget_aux = glade_xml_get_widget(xml, "entry_value");
	g_object_connect(G_OBJECT(widget_aux), "signal::changed", G_CALLBACK(on_entry_value_changed), xml, NULL);
	entry_completion = gtk_entry_completion_new ();
	gtk_entry_completion_set_text_column (entry_completion, 0);
	gtk_entry_completion_set_model (entry_completion, NULL);
	gtk_entry_completion_set_popup_completion (entry_completion, TRUE);
	gtk_entry_set_completion (GTK_ENTRY(widget_aux), entry_completion);

   	widget_aux = glade_xml_get_widget(xml, "cbox_tag_id");
	gtk_combo_box_set_active (GTK_COMBO_BOX(widget_aux), BMP_DATUM_ARTIST);

   	widget_aux = glade_xml_get_widget(xml, "cbox_sort");
	gtk_combo_box_set_active (GTK_COMBO_BOX(widget_aux), BMP_DATUM_ALBUM);

	widget_aux = glade_xml_get_widget(xml, "cbox_tag_id");
	g_object_connect(G_OBJECT(widget_aux), "signal::changed", G_CALLBACK(on_cbox_tag_id_changed), xml, NULL);

	widget_aux = glade_xml_get_widget(xml, "cbox_sort");
	g_object_connect(G_OBJECT(widget_aux), "signal::changed", G_CALLBACK(on_cbox_sort_changed), xml, NULL);

	widget_aux = glade_xml_get_widget(xml, "cb_exact");
	g_object_connect(G_OBJECT(widget_aux), "signal::toggled", G_CALLBACK(on_cb_exact_changed), xml, NULL);

#if 0
	/* Setup BROWSE Tab */
	widget_aux =
	    glade_xml_get_widget(xml, "tree_view_browse");

	gtk_tree_selection_set_mode
	    (gtk_tree_view_get_selection (GTK_TREE_VIEW(widget_aux)),
	     GTK_SELECTION_MULTIPLE);

#if 0
	g_object_connect(G_OBJECT(widget_aux), "signal::row-activated", G_CALLBACK(on_tree_view_search_row_activated), xml, NULL);
#endif

	/* Two columns, one for artist/album/title and a 2nd seperate column for track # */
	library_column_browse[0] =
		gtk_tree_view_column_new ();

	library_column_browse[1] =
		gtk_tree_view_column_new ();


	/* The cell renderer for the artist/album/title icon */
	library_renderer_browse[0] =
		gtk_cell_renderer_pixbuf_new ();

	g_object_set (library_renderer_browse[0], "width", 16, NULL);
	g_object_set (library_renderer_browse[0], "height", 16, NULL);

	gtk_tree_view_column_pack_start
		(library_column_browse[0],
		 library_renderer_browse[0],
		 FALSE);

	gtk_tree_view_column_set_attributes
		(library_column_browse[0],
		 library_renderer_browse[0],
		 "pixbuf",
		  0,
		  NULL);

	/* The cell renderer for the actual text */
	library_renderer_browse[1] =
		gtk_cell_renderer_text_new ();

	gtk_tree_view_column_pack_start
		(library_column_browse[0],
		 library_renderer_browse[1],
		 FALSE);

	gtk_tree_view_column_set_attributes
		(library_column_browse[0],
		 library_renderer_browse[1],
		 "text",
		  1,
		  NULL);

	gtk_tree_view_column_set_expand
		(library_column_browse[0],
		 FALSE);

#if 0

	// HERE CODE TO CREATE THE TRACK# COLUMN //

#endif

	gtk_tree_view_append_column
		(GTK_TREE_VIEW (widget_aux),
		 GTK_TREE_VIEW_COLUMN (library_column_browse[0]));

	widget_aux = glade_xml_get_widget(xml, "image_browse");
	gtk_image_set_from_file (GTK_IMAGE(widget_aux), DATA_DIR "/images/wait.gif");

	widget_aux = glade_xml_get_widget(xml, "browse_notebook");
	g_signal_connect (G_OBJECT(widget_aux), "realize",
                          G_CALLBACK(on_browse_notebook_realize),
                          xml);
#endif

	/* Common widgets */
	widget_aux = glade_xml_get_widget(xml, "button_enqueue");
	g_object_connect(G_OBJECT(widget_aux), "signal::clicked", G_CALLBACK(on_button_enqueue_clicked), xml, NULL);

	widget_aux = glade_xml_get_widget(xml, "button_play");
	g_object_connect(G_OBJECT(widget_aux), "signal::clicked", G_CALLBACK(on_button_play_clicked), xml, NULL);

	widget_aux = glade_xml_get_widget(xml, "button_close");
	g_object_connect(G_OBJECT(widget_aux), "swapped-signal::clicked", G_CALLBACK(gtk_widget_hide), dialog_window, NULL);
    }

    gtk_window_present (GTK_WINDOW(dialog_window));

}

void
bmp_ui_callback_stop_after_current_track (GtkToggleAction *action, gpointer data)
{
    gboolean active;

    active = gtk_toggle_action_get_active (action);

    bmp_system_control_stop_after_current_set (bmp_system_control, active, NULL);
}

/* Tracklist exporting */
typedef struct
{
	gchar *name;
	gchar *desc;
	gchar *suffix;
} ContainerTuple;


static void
export_find_containers (gpointer key, gpointer value, GList **containers)
{
    BmpPluginContainerInterface *container = (BmpPluginContainerInterface*)value;

    if (BMP_PLUGIN_CONTAINER_IFACE_GET_CLASS (container)->handle_write != NULL) 
      {
	ContainerTuple *tuple = g_new0 (ContainerTuple,1);	
	const gchar** info;

	info = bmp_plugin_container_get_export_data (container);

	tuple->name = g_strdup (key);
	tuple->desc = g_strdup (info[0]);
	tuple->suffix = g_strdup (info[1]);
    
	*containers = g_list_append (*containers, tuple); 
      }
}

static gboolean
export_run_fc (GtkWidget *button, gpointer data)
{
    GladeXML *xml = (GladeXML*)data;
    GtkFileChooserDialog *dialog = glade_xml_get_widget_warn (xml, "fc_dialog"); 
    GtkWidget *widget;
    gint rv = 0;

    gtk_widget_realize (dialog);

    widget =
	glade_xml_get_widget_warn (xml, "entry_filename");

    if (strlen(gtk_entry_get_text (GTK_ENTRY(widget)))>0)
      {
	gchar *basename, *folder;

	basename =
	  g_path_get_basename (gtk_entry_get_text (GTK_ENTRY(widget)));
	folder =
	  g_path_get_dirname (gtk_entry_get_text (GTK_ENTRY(widget)));

	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(dialog), folder); 
	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER(dialog), basename);
      
	g_free (basename);
	g_free (folder);
      }
  
    rv = gtk_dialog_run (GTK_DIALOG(dialog));

    switch (rv)
      {
	case GTK_RESPONSE_CANCEL:
	  {
	  }
	break;

	case GTK_RESPONSE_OK:
	  {
	    gchar *filename;
	    filename =
		gtk_file_chooser_get_filename (GTK_FILE_CHOOSER(dialog));
	    gtk_entry_set_text (GTK_ENTRY(widget), filename); 
	    g_free (filename);
	  }
      }

    gtk_widget_hide (GTK_WIDGET(dialog));

    return FALSE;
}

static void
export_tree_view_cursor_changed (GtkTreeView *tree_view,
				 gpointer user_data)
{
  GtkTreeSelection *selection;
  GtkTreeModel *model;
  GtkTreeIter iter;
  GtkWidget *entry;
  GString *result;
  GladeXML *xml = (GladeXML*)user_data;
  gchar *composited = NULL, *suffix = NULL;
  gint position;

  selection = 
      gtk_tree_view_get_selection (tree_view);
  entry =
      glade_xml_get_widget_warn (xml, "entry_filename");
  result =
      g_string_new (gtk_entry_get_text (GTK_ENTRY(entry)));
  
  position = strlen (result->str)-1;

  while ( (result->str[position] != '.') && position >= 0)
    {
      position--;
    }

  if (position != 0)
      g_string_truncate (result, position);
  
  gtk_tree_selection_get_selected (selection, &model, &iter);
  gtk_tree_model_get (model, &iter, 1, &suffix, -1);
  composited = g_strconcat (result->str, ".", suffix, NULL); 

  gtk_entry_set_text (GTK_ENTRY(entry), composited);
}

void
bmp_ui_callback_export_tracklist (GtkAction *action, gpointer data)
{
  static GladeXML *xml = NULL;
  static GtkWidget *dialog_window = NULL;
  static GtkWidget *tree_view = NULL;

  const gchar *glade_file = DATA_DIR "/glade/dialog_export_tracklist.glade";

  GtkTreeSelection *selection;
  GtkTreeModel *store;
  GtkTreeIter iter;
  gchar *listname = NULL, *suffix = NULL, *suffix_fq = NULL;
  gint rv = 0;

  if (! dialog_window)
    {
	GtkTreeIter iter;
	GtkListStore *store = NULL;
	GtkWidget *widget = NULL;
	GtkTreeViewColumn *column = NULL;
	GtkCellRenderer *renderer = NULL;
	GList *containers = NULL;

	xml =
	    glade_xml_new_or_die(Q_("Dialog"), glade_file, NULL, NULL);
	dialog_window =
	    glade_xml_get_widget(xml, "export_dialog");
	tree_view =
	    glade_xml_get_widget(xml, "treeview_plugins");
	column =
	    gtk_tree_view_column_new ();
	renderer =
	    gtk_cell_renderer_text_new ();

	gtk_tree_view_column_pack_start
		(column,
		 renderer,
		 FALSE);

	gtk_tree_view_column_set_attributes
		(column,
		 renderer, 
		 "text",
		  0,
		  NULL);

	gtk_tree_view_column_set_expand
		(column,
		 FALSE);

	gtk_tree_view_column_set_title
		(column,
		 N_("Export Plugin")); 

	gtk_tree_view_append_column
		(GTK_TREE_VIEW (tree_view),
		 GTK_TREE_VIEW_COLUMN (column));


	g_hash_table_foreach (plugins_table[BMP_PLUGIN_TYPE_CONTAINER], (GHFunc)export_find_containers, &containers);
    
	store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
	
	for ( ;containers;containers=containers->next)
	  {
	    ContainerTuple *tuple = (ContainerTuple*)containers->data;

	    gtk_list_store_append (GTK_LIST_STORE(store), &iter);
	    gtk_list_store_set (GTK_LIST_STORE(store), &iter, 0, tuple->desc, 1, tuple->suffix, 2, tuple->name, -1);
	  }

	gtk_tree_view_set_model (GTK_TREE_VIEW(tree_view), GTK_TREE_MODEL(store));

	gtk_widget_realize (dialog_window);

	g_object_connect (G_OBJECT(glade_xml_get_widget_warn(xml, "button_browse")), "signal::clicked", export_run_fc, xml, NULL);
	g_object_connect (G_OBJECT(tree_view), "signal::cursor-changed", export_tree_view_cursor_changed, xml, NULL);
    }
  else
    {
      store = 
	  gtk_tree_view_get_model (GTK_TREE_VIEW(tree_view));
      selection =
	  gtk_tree_view_get_selection (GTK_TREE_VIEW(tree_view));
      gtk_tree_model_get_iter_first (store, &iter);
      gtk_tree_selection_select_iter (selection, &iter);
    }

  store = 
      gtk_tree_view_get_model (GTK_TREE_VIEW(tree_view));

  gtk_tree_model_get_iter_first (store, &iter);
  gtk_tree_model_get (store, &iter, 1, &suffix, -1);

  suffix_fq = g_strconcat (".", suffix, NULL);

  listname = g_strconcat (g_get_home_dir(), 
		     G_DIR_SEPARATOR_S,
		     bmp_playlist_get_nth_tracklist_name (bmp_playlist, 
							  g_object_get_int (bmp_system_control, "current_tracklist")),
		     suffix_fq,
		     NULL);
  g_free (suffix_fq);
  gtk_entry_set_text (GTK_ENTRY(glade_xml_get_widget_warn (xml, "entry_filename")), listname);

  rv = gtk_dialog_run (GTK_DIALOG(dialog_window)); 

  switch (rv)
    {
	case GTK_RESPONSE_CANCEL: break;

	case GTK_RESPONSE_OK:
	  {
	    GtkListStore *store;
	    GtkTreeIter iter;
	    gchar *container;
	    gboolean rv;

	    gtk_tree_selection_get_selected (gtk_tree_view_get_selection (GTK_TREE_VIEW(glade_xml_get_widget_warn (xml, "treeview_plugins"))), &store, &iter);	    
	    gtk_tree_model_get (GTK_TREE_MODEL(store), &iter, 2, &container, -1);

	    rv = bmp_system_control_export_tracklist (bmp_system_control,
						      container,
						      g_object_get_int (bmp_system_control, "current_tracklist"),
						      !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget_warn(xml, "rb_options1"))),
						      gtk_entry_get_text (GTK_ENTRY(glade_xml_get_widget_warn (xml, "entry_filename"))),
						      NULL);  

	    if (!rv)
	      {
		// ERROR HERE	
	      }
	  }
    }

  gtk_widget_hide (dialog_window);
}

